@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2007 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2007-2008 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -11,13 +11,15 @@
 
 #include "katze-throbber.h"
 
-#include <gtk/gtk.h>
+#include "katze-utils.h"
+
 #include <glib/gi18n.h>
+#include <math.h>
 
-G_DEFINE_TYPE (KatzeThrobber, katze_throbber, GTK_TYPE_MISC)
-
-struct _KatzeThrobberPrivate
+struct _KatzeThrobber
 {
+    GtkMisc parent_instance;
+
     GtkIconSize icon_size;
     gchar* icon_name;
     GdkPixbuf* pixbuf;
@@ -33,8 +35,7 @@ struct _KatzeThrobberPrivate
     gint height;
 };
 
-#define KATZE_THROBBER_GET_PRIVATE(obj) \
-    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KATZE_TYPE_THROBBER, KatzeThrobberPrivate))
+G_DEFINE_TYPE (KatzeThrobber, katze_throbber, GTK_TYPE_MISC);
 
 enum
 {
@@ -50,7 +51,7 @@ enum
 };
 
 static void
-katze_throbber_dispose(GObject* object);
+katze_throbber_dispose (GObject* object);
 
 static void
 katze_throbber_set_property (GObject* object,
@@ -132,7 +133,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_int (
                                      "icon-size",
                                      "Icon size",
-                                     _("Symbolic size to use for the animation"),
+                                     "Symbolic size to use for the animation",
                                      0, G_MAXINT, GTK_ICON_SIZE_MENU,
                                      flags));
 
@@ -141,7 +142,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_string (
                                      "icon-name",
                                      "Icon Name",
-                                     _("The name of an icon containing animation frames"),
+                                     "The name of an icon containing animation frames",
                                      "process-working",
                                      flags));
 
@@ -150,7 +151,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_object (
                                      "pixbuf",
                                      "Pixbuf",
-                                     _("A GdkPixbuf containing animation frames"),
+                                     "A GdkPixbuf containing animation frames",
                                      GDK_TYPE_PIXBUF,
                                      G_PARAM_READWRITE));
 
@@ -159,7 +160,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_boolean (
                                      "animated",
                                      "Animated",
-                                     _("Whether the throbber should be animated"),
+                                     "Whether the throbber should be animated",
                                      FALSE,
                                      flags));
 
@@ -168,7 +169,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_string (
                                      "static-icon-name",
                                      "Static Icon Name",
-                                     _("The name of an icon to be used as the static image"),
+                                     "The name of an icon to be used as the static image",
                                      NULL,
                                      flags));
 
@@ -177,7 +178,7 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_object (
                                      "static-pixbuf",
                                      "Static Pixbuf",
-                                     _("A GdkPixbuf to be used as the static image"),
+                                     "A GdkPixbuf to be used as the static image",
                                      GDK_TYPE_PIXBUF,
                                      G_PARAM_READWRITE));
 
@@ -186,11 +187,9 @@ katze_throbber_class_init (KatzeThrobberClass* class)
                                      g_param_spec_string (
                                      "static-stock-id",
                                      "Static Stock ID",
-                                     _("The stock ID of an icon to be used as the static image"),
+                                     "The stock ID of an icon to be used as the static image",
                                      NULL,
                                      flags));
-
-    g_type_class_add_private (object_class, sizeof (KatzeThrobberPrivate));
 }
 
 static void
@@ -198,20 +197,16 @@ katze_throbber_init (KatzeThrobber *throbber)
 {
     GTK_WIDGET_SET_FLAGS (throbber, GTK_NO_WINDOW);
 
-    throbber->priv = KATZE_THROBBER_GET_PRIVATE (throbber);
-
-    KatzeThrobberPrivate* priv = throbber->priv;
-    priv->timer_id = -1;
+    throbber->timer_id = -1;
 }
 
 static void
 katze_throbber_dispose (GObject* object)
 {
     KatzeThrobber* throbber = KATZE_THROBBER (object);
-    KatzeThrobberPrivate* priv = throbber->priv;
 
-    if (G_UNLIKELY (priv->timer_id >= 0))
-        g_source_remove (priv->timer_id);
+    if (G_UNLIKELY (throbber->timer_id >= 0))
+        g_source_remove (throbber->timer_id);
 
     (*G_OBJECT_CLASS (katze_throbber_parent_class)->dispose) (object);
 }
@@ -220,13 +215,12 @@ static void
 katze_throbber_destroy (GtkObject* object)
 {
     KatzeThrobber* throbber = KATZE_THROBBER (object);
-    KatzeThrobberPrivate* priv = throbber->priv;
 
-    katze_assign (priv->icon_name, NULL);
-    katze_object_assign (priv->pixbuf, NULL);
-    katze_assign (priv->static_icon_name, NULL);
-    katze_object_assign (priv->static_pixbuf, NULL);
-    katze_assign (priv->static_stock_id, NULL);
+    katze_assign (throbber->icon_name, NULL);
+    katze_object_assign (throbber->pixbuf, NULL);
+    katze_assign (throbber->static_icon_name, NULL);
+    katze_object_assign (throbber->static_pixbuf, NULL);
+    katze_assign (throbber->static_stock_id, NULL);
 
     GTK_OBJECT_CLASS (katze_throbber_parent_class)->destroy (object);
 }
@@ -269,10 +263,10 @@ katze_throbber_set_property (GObject*      object,
 }
 
 static void
-katze_throbber_get_property (GObject*     object,
-                             guint        prop_id,
-                             GValue*      value,
-                             GParamSpec*  pspec)
+katze_throbber_get_property (GObject*    object,
+                             guint       prop_id,
+                             GValue*     value,
+                             GParamSpec* pspec)
 {
     KatzeThrobber* throbber = KATZE_THROBBER (object);
 
@@ -334,16 +328,20 @@ void
 katze_throbber_set_icon_size (KatzeThrobber* throbber,
                               GtkIconSize    icon_size)
 {
+    GtkSettings* gtk_settings;
+
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
+    gtk_settings = gtk_widget_get_settings (GTK_WIDGET (throbber));
+    g_return_if_fail (gtk_icon_size_lookup_for_settings (gtk_settings,
+                                                         icon_size,
+                                                         &throbber->width,
+                                                         &throbber->height));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    g_return_if_fail (gtk_icon_size_lookup (icon_size,
-                                            &priv->width,
-                                            &priv->height));
-
-    priv->icon_size = icon_size;
+    throbber->icon_size = icon_size;
 
     g_object_notify (G_OBJECT (throbber), "icon-size");
+
+    gtk_widget_queue_draw (GTK_WIDGET (throbber));
 }
 
 /**
@@ -361,8 +359,7 @@ katze_throbber_set_icon_name (KatzeThrobber*  throbber,
 {
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    katze_assign (priv->icon_name, g_strdup (icon_name));
+    katze_assign (throbber->icon_name, g_strdup (icon_name));
 
     if (icon_name)
         icon_theme_changed (throbber);
@@ -386,21 +383,24 @@ katze_throbber_set_pixbuf (KatzeThrobber* throbber,
                            GdkPixbuf*     pixbuf)
 {
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
-    g_return_if_fail (pixbuf == NULL || GDK_IS_PIXBUF (pixbuf));
+    g_return_if_fail (!pixbuf || GDK_IS_PIXBUF (pixbuf));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    katze_object_assign (priv->pixbuf, pixbuf);
+    katze_object_assign (throbber->pixbuf, pixbuf);
+
+    g_object_freeze_notify (G_OBJECT (throbber));
 
     if (pixbuf)
     {
         g_object_ref (pixbuf);
 
-        katze_assign (priv->icon_name, NULL);
+        katze_assign (throbber->icon_name, NULL);
+        g_object_notify (G_OBJECT (throbber), "icon-name");
     }
 
     gtk_widget_queue_draw (GTK_WIDGET (throbber));
 
     g_object_notify (G_OBJECT (throbber), "pixbuf");
+    g_object_thaw_notify (G_OBJECT (throbber));
 }
 
 /**
@@ -416,14 +416,13 @@ katze_throbber_set_animated (KatzeThrobber*  throbber,
 {
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    if (G_UNLIKELY (priv->animated == animated))
+    if (G_UNLIKELY (throbber->animated == animated))
         return;
 
-    priv->animated = animated;
+    throbber->animated = animated;
 
-    if (animated && (priv->timer_id < 0))
-        priv->timer_id = g_timeout_add_full (
+    if (animated && (throbber->timer_id < 0))
+        throbber->timer_id = g_timeout_add_full (
                          G_PRIORITY_LOW, 50,
                          (GSourceFunc)katze_throbber_timeout,
                          throbber,
@@ -449,17 +448,22 @@ katze_throbber_set_static_icon_name (KatzeThrobber*  throbber,
 {
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    katze_assign (priv->static_icon_name, g_strdup (icon_name));
+    katze_assign (throbber->static_icon_name, g_strdup (icon_name));
+
+    g_object_freeze_notify (G_OBJECT (throbber));
 
     if (icon_name)
     {
-        katze_assign (priv->static_stock_id, NULL);
+        katze_assign (throbber->static_stock_id, NULL);
 
         icon_theme_changed (throbber);
+
+        g_object_notify (G_OBJECT (throbber), "static-pixbuf");
+        g_object_notify (G_OBJECT (throbber), "static-stock-id");
     }
 
     g_object_notify (G_OBJECT (throbber), "static-icon-name");
+    g_object_thaw_notify (G_OBJECT (throbber));
 }
 
 /**
@@ -480,18 +484,25 @@ katze_throbber_set_static_pixbuf (KatzeThrobber* throbber,
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
     g_return_if_fail (pixbuf == NULL || GDK_IS_PIXBUF (pixbuf));
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    katze_object_assign (priv->static_pixbuf, pixbuf);
+    katze_object_assign (throbber->static_pixbuf, pixbuf);
+
+    g_object_freeze_notify (G_OBJECT (throbber));
 
     if (pixbuf)
     {
         g_object_ref (pixbuf);
 
-        katze_assign (priv->static_icon_name, NULL);
-        katze_assign (priv->static_stock_id, NULL);
+        katze_assign (throbber->static_icon_name, NULL);
+        katze_assign (throbber->static_stock_id, NULL);
+
+        gtk_widget_queue_draw (GTK_WIDGET (throbber));
+
+        g_object_notify (G_OBJECT (throbber), "static-icon-name");
+        g_object_notify (G_OBJECT (throbber), "static-stock-id");
     }
 
     g_object_notify (G_OBJECT (throbber), "static-pixbuf");
+    g_object_thaw_notify (G_OBJECT (throbber));
 }
 
 /**
@@ -501,27 +512,32 @@ katze_throbber_set_static_pixbuf (KatzeThrobber* throbber,
  *
  * Sets the stock ID of an icon that should provide the static image.
  *
- * The statc icon name and pixbuf are automatically invalidated.
+ * The static icon name and pixbuf are automatically invalidated.
  **/
 void
-katze_throbber_set_static_stock_id (KatzeThrobber*  throbber,
-                                    const gchar*    stock_id)
+katze_throbber_set_static_stock_id (KatzeThrobber* throbber,
+                                    const gchar*   stock_id)
 {
     g_return_if_fail (KATZE_IS_THROBBER (throbber));
 
+    g_object_freeze_notify (G_OBJECT (throbber));
+
     if (stock_id)
     {
         GtkStockItem stock_item;
         g_return_if_fail (gtk_stock_lookup (stock_id, &stock_item));
+
+        g_object_notify (G_OBJECT (throbber), "static-icon-name");
+        g_object_notify (G_OBJECT (throbber), "static-pixbuf");
     }
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    katze_assign (priv->static_stock_id, g_strdup (stock_id));
+    katze_assign (throbber->static_stock_id, g_strdup (stock_id));
 
     if (stock_id)
         icon_theme_changed (throbber);
 
     g_object_notify (G_OBJECT (throbber), "static-stock-id");
+    g_object_thaw_notify (G_OBJECT (throbber));
 }
 
 /**
@@ -537,8 +553,7 @@ katze_throbber_get_icon_size (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), GTK_ICON_SIZE_INVALID);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->icon_size;
+    return throbber->icon_size;
 }
 
 /**
@@ -554,8 +569,7 @@ katze_throbber_get_icon_name (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), NULL);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->icon_name;
+    return throbber->icon_name;
 }
 
 /**
@@ -573,8 +587,7 @@ katze_throbber_get_pixbuf (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), NULL);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->pixbuf;
+    return throbber->pixbuf;
 }
 
 /**
@@ -590,8 +603,7 @@ katze_throbber_get_animated (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), FALSE);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->animated;
+    return throbber->animated;
 }
 
 /**
@@ -608,8 +620,7 @@ katze_throbber_get_static_icon_name (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), NULL);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->static_icon_name;
+    return throbber->static_icon_name;
 }
 
 /**
@@ -627,8 +638,7 @@ katze_throbber_get_static_pixbuf (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), NULL);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->static_pixbuf;
+    return throbber->static_pixbuf;
 }
 
 /**
@@ -645,8 +655,7 @@ katze_throbber_get_static_stock_id (KatzeThrobber* throbber)
 {
     g_return_val_if_fail (KATZE_IS_THROBBER (throbber), NULL);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-    return priv->static_stock_id;
+    return throbber->static_stock_id;
 }
 
 static void
@@ -662,10 +671,6 @@ katze_throbber_unrealize (GtkWidget* widget)
 {
     if (GTK_WIDGET_CLASS (katze_throbber_parent_class)->unrealize)
         GTK_WIDGET_CLASS (katze_throbber_parent_class)->unrealize (widget);
-
-    KatzeThrobberPrivate* priv = KATZE_THROBBER(widget)->priv;
-    katze_object_assign (priv->pixbuf, NULL);
-    katze_object_assign (priv->static_pixbuf, NULL);
 }
 
 static void
@@ -673,16 +678,17 @@ pixbuf_assign_icon (GdkPixbuf**    pixbuf,
                     const gchar*   icon_name,
                     KatzeThrobber* throbber)
 {
+    GdkScreen* screen;
+    GtkIconTheme* icon_theme;
+
     if (*pixbuf)
         g_object_unref (*pixbuf);
 
-    KatzeThrobberPrivate* priv = throbber->priv;
-
-    GdkScreen* screen = gtk_widget_get_screen (GTK_WIDGET (throbber));
-    GtkIconTheme* icon_theme = gtk_icon_theme_get_for_screen (screen);
+    screen = gtk_widget_get_screen (GTK_WIDGET (throbber));
+    icon_theme = gtk_icon_theme_get_for_screen (screen);
     *pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                         icon_name,
-                                        MAX (priv->width, priv->height),
+                                        MAX (throbber->width, throbber->height),
                                         (GtkIconLookupFlags) 0,
                                         NULL);
 }
@@ -690,25 +696,24 @@ pixbuf_assign_icon (GdkPixbuf**    pixbuf,
 static void
 icon_theme_changed (KatzeThrobber* throbber)
 {
-    KatzeThrobberPrivate* priv = throbber->priv;
+    if (throbber->icon_name)
+        pixbuf_assign_icon (&throbber->pixbuf,
+                            throbber->icon_name, throbber);
 
-    if (priv->icon_name)
-        pixbuf_assign_icon (&priv->pixbuf, priv->icon_name,
-                            throbber);
+    if (throbber->static_icon_name)
+        pixbuf_assign_icon (&throbber->static_pixbuf,
+                            throbber->static_icon_name, throbber);
+    else if (throbber->static_stock_id)
+        katze_object_assign (throbber->static_pixbuf,
+            gtk_widget_render_icon (GTK_WIDGET (throbber),
+                                    throbber->static_stock_id,
+                                    throbber->icon_size,
+                                    NULL));
 
-    if (priv->static_icon_name)
-        pixbuf_assign_icon (&priv->static_pixbuf, priv->static_icon_name,
-                            throbber);
-    else if (priv->static_stock_id)
-    {
-        if (priv->static_pixbuf)
-            g_object_unref (priv->static_pixbuf);
-
-        priv->static_pixbuf = gtk_widget_render_icon (GTK_WIDGET (throbber),
-                                                      priv->static_stock_id,
-                                                      priv->icon_size,
-                                                      NULL);
-    }
+    g_object_freeze_notify (G_OBJECT (throbber));
+    g_object_notify (G_OBJECT (throbber), "pixbuf");
+    g_object_notify (G_OBJECT (throbber), "static-pixbuf");
+    g_object_thaw_notify (G_OBJECT (throbber));
 
     gtk_widget_queue_draw (GTK_WIDGET (throbber));
 }
@@ -729,21 +734,17 @@ katze_throbber_unmap (GtkWidget* widget)
 static gboolean
 katze_throbber_timeout (KatzeThrobber*  throbber)
 {
-    KatzeThrobberPrivate* priv = throbber->priv;
-
-    priv->index++;
+    throbber->index++;
     gtk_widget_queue_draw (GTK_WIDGET (throbber));
 
-    return priv->animated;
+    return throbber->animated;
 }
 
 static void
 katze_throbber_timeout_destroy (KatzeThrobber*  throbber)
 {
-    KatzeThrobberPrivate* priv = throbber->priv;
-
-    priv->index = 0;
-    priv->timer_id = -1;
+    throbber->index = 0;
+    throbber->timer_id = -1;
 }
 
 static void
@@ -773,105 +774,144 @@ static void
 katze_throbber_size_request (GtkWidget*      widget,
                              GtkRequisition* requisition)
 {
-    KatzeThrobberPrivate* priv = KATZE_THROBBER (widget)->priv;
+    KatzeThrobber* throbber = KATZE_THROBBER (widget);
 
-    requisition->width = priv->width;
-    requisition->height = priv->height;
+    requisition->width = throbber->width;
+    requisition->height = throbber->height;
 
     GTK_WIDGET_CLASS (katze_throbber_parent_class)->size_request (widget,
                                                                   requisition);
 }
 
+static void
+katze_throbber_aligned_coords (GtkWidget* widget,
+                               gint*      ax,
+                               gint*      ay)
+{
+    gfloat xalign, yalign;
+    gint xpad, ypad;
+
+    gtk_misc_get_alignment (GTK_MISC (widget), &xalign, &yalign);
+    if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+        xalign = 1.0f - xalign;
+    gtk_misc_get_padding (GTK_MISC (widget), &xpad, &ypad);
+
+    *ax = floor (widget->allocation.x + xpad
+        + ((widget->allocation.width - widget->requisition.width) * xalign));
+    *ay = floor (widget->allocation.y + ypad
+        + ((widget->allocation.height - widget->requisition.height) * yalign));
+}
+
 static gboolean
 katze_throbber_expose_event (GtkWidget*      widget,
                              GdkEventExpose* event)
 {
-    KatzeThrobberPrivate* priv = KATZE_THROBBER (widget)->priv;
+    gint ax, ay;
+    KatzeThrobber* throbber = KATZE_THROBBER (widget);
 
-    if (G_UNLIKELY (!priv->width || !priv->height))
+    if (G_UNLIKELY (!throbber->width || !throbber->height))
         return TRUE;
 
-    if (G_UNLIKELY (!priv->pixbuf && !priv->static_pixbuf))
-        if (priv->animated && !priv->pixbuf && !priv->icon_name)
+    if (G_UNLIKELY (!throbber->pixbuf && !throbber->static_pixbuf))
+        if (throbber->animated && !throbber->pixbuf && !throbber->icon_name)
             return TRUE;
 
-    if (!priv->animated &&
-        (priv->static_pixbuf || priv->static_icon_name || priv->static_stock_id))
+    if (!throbber->animated && (throbber->static_pixbuf
+        || throbber->static_icon_name || throbber->static_stock_id))
     {
-        if (G_UNLIKELY (!priv->static_pixbuf && priv->static_icon_name))
+        if (G_UNLIKELY (!throbber->static_pixbuf && throbber->static_icon_name))
         {
             icon_theme_changed (KATZE_THROBBER (widget));
 
-            if (!priv->static_pixbuf)
+            if (!throbber->static_pixbuf)
             {
-                g_warning ("Named icon '%s' couldn't be loaded",
-                           priv->static_icon_name);
-                katze_assign (priv->static_icon_name, NULL);
+                g_warning (_("Named icon '%s' couldn't be loaded"),
+                           throbber->static_icon_name);
+                katze_assign (throbber->static_icon_name, NULL);
+                g_object_notify (G_OBJECT (throbber), "static-icon-name");
                 return TRUE;
             }
         }
-        else if (G_UNLIKELY (!priv->static_pixbuf && priv->static_stock_id))
+        else if (G_UNLIKELY (!throbber->static_pixbuf && throbber->static_stock_id))
         {
             icon_theme_changed (KATZE_THROBBER (widget));
 
-            if (!priv->static_pixbuf)
+            if (!throbber->static_pixbuf)
             {
-                g_warning ("Stock icon '%s' couldn't be loaded",
-                           priv->static_stock_id);
-                katze_assign (priv->static_stock_id, NULL);
+                g_warning (_("Stock icon '%s' couldn't be loaded"),
+                           throbber->static_stock_id);
+                katze_assign (throbber->static_stock_id, NULL);
+                g_object_notify (G_OBJECT (throbber), "static-stock-id");
                 return TRUE;
             }
         }
 
-        gdk_draw_pixbuf (event->window, NULL, priv->static_pixbuf,
-                         0, 0,
-                         widget->allocation.x,
-                         widget->allocation.y,
-                         priv->width, priv->height,
+        katze_throbber_aligned_coords (widget, &ax, &ay);
+
+        gdk_draw_pixbuf (event->window, NULL, throbber->static_pixbuf,
+                         0, 0, ax, ay,
+                         throbber->width, throbber->height,
                          GDK_RGB_DITHER_NONE, 0, 0);
     }
     else
     {
-        if (G_UNLIKELY (priv->icon_name && !priv->pixbuf))
+        if (G_UNLIKELY (throbber->icon_name && !throbber->pixbuf))
         {
             icon_theme_changed (KATZE_THROBBER (widget));
 
-            if (!priv->pixbuf)
+            if (!throbber->pixbuf)
             {
-                g_warning ("Icon '%s' couldn't be loaded", priv->icon_name);
-                katze_assign (priv->icon_name, NULL);
+                /* Fallback to a stock icon */
+                katze_assign (throbber->icon_name, g_strdup (GTK_STOCK_EXECUTE));
+                g_object_notify (G_OBJECT (throbber), "icon-name");
                 return TRUE;
             }
         }
 
-        if (G_UNLIKELY (!priv->pixbuf))
-            return TRUE;
+        gint cols = gdk_pixbuf_get_width (throbber->pixbuf) / throbber->width;
+        gint rows = gdk_pixbuf_get_height (throbber->pixbuf) / throbber->height;
 
-        gint cols = gdk_pixbuf_get_width (priv->pixbuf) / priv->width;
-        gint rows = gdk_pixbuf_get_height (priv->pixbuf) / priv->height;
+        if (G_UNLIKELY (cols == 1 && cols == rows))
+        {
+            katze_throbber_aligned_coords (widget, &ax, &ay);
+
+            if (throbber->animated)
+                gdk_draw_pixbuf (event->window, NULL, throbber->pixbuf,
+                                 0, 0, ax, ay,
+                                 throbber->width, throbber->height,
+                                 GDK_RGB_DITHER_NONE, 0, 0);
+            return TRUE;
+        }
 
         if (G_LIKELY (cols > 0 && rows > 0))
         {
-            gint index = priv->index % (cols * rows);
+            gint idx;
+            guint x, y;
 
-            if (G_LIKELY (priv->timer_id >= 0))
-                index = MAX (index, 1);
+            katze_throbber_aligned_coords (widget, &ax, &ay);
 
-            guint x = (index % cols) * priv->width;
-            guint y = (index / cols) * priv->height;
+            idx = throbber->index % (cols * rows);
+            if (G_LIKELY (throbber->timer_id >= 0))
+                idx = MAX (idx, 1);
 
-            gdk_draw_pixbuf (event->window, NULL, priv->pixbuf,
-                             x, y,
-                             widget->allocation.x,
-                             widget->allocation.y,
-                             priv->width, priv->height,
+            x = (idx % cols) * throbber->width;
+            y = (idx / cols) * throbber->height;
+
+            gdk_draw_pixbuf (event->window, NULL, throbber->pixbuf,
+                             x, y, ax, ay,
+                             throbber->width, throbber->height,
                              GDK_RGB_DITHER_NONE, 0, 0);
         }
         else
         {
-            g_warning ("Animation frames are broken");
-            katze_assign (priv->icon_name, NULL);
-            katze_object_assign (priv->pixbuf, NULL);
+            g_warning (_("Animation frames are broken"));
+            katze_assign (throbber->icon_name, NULL);
+            katze_object_assign (throbber->pixbuf, NULL);
+
+            g_object_freeze_notify (G_OBJECT (throbber));
+            g_object_notify (G_OBJECT (throbber), "icon-name");
+            g_object_notify (G_OBJECT (throbber), "pixbuf");
+            g_object_thaw_notify (G_OBJECT (throbber));
         }
     }
 
