@@ -32,6 +32,10 @@ midori_browser_edit_bookmark_dialog_new (MidoriBrowser* browser,
                                          gboolean       new_bookmark,
                                          gboolean       is_folder);
 
+void
+midori_browser_open_bookmark (MidoriBrowser* browser,
+                              KatzeItem*     item);
+
 struct _MidoriBookmarks
 {
     GtkVBox parent_instance;
@@ -182,6 +186,14 @@ midori_bookmarks_folder_clicked_cb (GtkWidget* toolitem)
                                              NULL, TRUE, TRUE);
 }
 
+static void
+midori_bookmarks_import_clicked_cb (GtkWidget* toolitem)
+{
+    MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (toolitem));
+    /* FIXME: Take selected folder into account */
+    midori_browser_activate_action (browser, "BookmarksImport");
+}
+
 static void
 midori_bookmarks_cursor_or_row_changed_cb (GtkTreeView*     treeview,
                                            MidoriBookmarks* bookmarks)
@@ -255,6 +267,13 @@ midori_bookmarks_get_toolbar (MidoriViewable* viewable)
         gtk_tool_item_set_expand (toolitem, TRUE);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
         gtk_widget_show (GTK_WIDGET (toolitem));
+        toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_CONVERT);
+        gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
+                                     _("Import bookmarks..."));
+        g_signal_connect (toolitem, "clicked",
+            G_CALLBACK (midori_bookmarks_import_clicked_cb), bookmarks);
+        gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
+        gtk_widget_show (GTK_WIDGET (toolitem));
         toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_DIRECTORY);
         gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
                                      _("Add a new folder"));
@@ -563,9 +582,7 @@ midori_bookmarks_treeview_render_icon_cb (GtkTreeViewColumn* column,
         pixbuf = gtk_widget_render_icon (treeview, GTK_STOCK_DIRECTORY,
                                          GTK_ICON_SIZE_MENU, NULL);
     else if (katze_item_get_uri (item))
-        pixbuf = katze_net_load_icon (
-            MIDORI_BOOKMARKS (gtk_widget_get_parent (treeview))->net,
-            katze_item_get_uri (item), NULL, treeview, NULL);
+        pixbuf = katze_load_cached_icon (katze_item_get_uri (item), treeview);
     g_object_set (renderer, "pixbuf", pixbuf, NULL);
 
     if (pixbuf)
@@ -603,19 +620,17 @@ midori_bookmarks_row_activated_cb (GtkTreeView*       treeview,
     GtkTreeModel* model;
     GtkTreeIter iter;
     KatzeItem* item;
-    const gchar* uri;
 
     model = gtk_tree_view_get_model (treeview);
 
     if (gtk_tree_model_get_iter (model, &iter, path))
     {
+        MidoriBrowser* browser;
+
         gtk_tree_model_get (model, &iter, 0, &item, -1);
-        uri = katze_item_get_uri (item);
-        if (uri && *uri)
-        {
-            MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (bookmarks));
-            midori_browser_set_current_uri (browser, uri);
-        }
+
+        browser = midori_browser_get_for_widget (GTK_WIDGET (bookmarks));
+        midori_browser_open_bookmark (browser, item);
 
         g_object_unref (item);
     }
@@ -797,8 +812,7 @@ midori_bookmarks_popup (GtkWidget*       widget,
     midori_bookmarks_popup_item (menu, GTK_STOCK_DELETE, NULL,
         item, midori_bookmarks_delete_activate_cb, bookmarks);
 
-    sokoke_widget_popup (widget, GTK_MENU (menu),
-                         event, SOKOKE_MENU_POSITION_CURSOR);
+    katze_widget_popup (widget, GTK_MENU (menu), event, KATZE_MENU_POSITION_CURSOR);
 }
 
 static gboolean
@@ -935,6 +949,7 @@ midori_bookmarks_finalize (GObject* object)
 {
     MidoriBookmarks* bookmarks = MIDORI_BOOKMARKS (object);
 
+    midori_bookmarks_disconnect_folder (bookmarks, bookmarks->array);
     if (bookmarks->app)
         g_object_unref (bookmarks->app);
     g_object_unref (bookmarks->net);
