@@ -73,7 +73,7 @@ midori_extensions_class_init (MidoriExtensionsClass* class)
     gobject_class->get_property = midori_extensions_get_property;
     gobject_class->finalize = midori_extensions_finalize;
 
-    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
+    flags = G_PARAM_READWRITE;
 
     g_object_class_install_property (gobject_class,
                                      PROP_APP,
@@ -183,6 +183,10 @@ midori_extensions_set_property (GObject*      object,
         i = 0;
         while ((extension = katze_array_get_nth_item (array, i++)))
             midori_extensions_add_item_cb (array, extension, extensions);
+
+        /* Hide if there are no extensions at all */
+        if (!katze_array_get_nth_item (array, 0))
+            gtk_widget_hide (GTK_WIDGET (object));
     }
         break;
     default:
@@ -221,7 +225,10 @@ midori_extensions_treeview_render_tick_cb (GtkTreeViewColumn* column,
 
     gtk_tree_model_get (model, iter, 0, &extension, -1);
 
-    g_object_set (renderer, "active", midori_extension_is_active (extension), NULL);
+    g_object_set (renderer,
+        "activatable", midori_extension_is_prepared (extension),
+        "active", midori_extension_is_active (extension),
+        NULL);
 
     g_object_unref (extension);
 }
@@ -244,12 +251,16 @@ midori_extensions_treeview_render_text_cb (GtkTreeViewColumn* column,
     name = katze_object_get_string (extension, "name");
     version = katze_object_get_string (extension, "version");
     desc = katze_object_get_string (extension, "description");
-    text = g_markup_printf_escaped ("<b>%s</b> %s\n%s", name, version, desc);
+    text = g_markup_printf_escaped ("<b>%s</b> %s\n%s",
+        name, version && *version ? version : "", desc);
     g_free (name);
     g_free (version);
     g_free (desc);
 
-    g_object_set (renderer, "markup", text, NULL);
+    g_object_set (renderer,
+        "markup", text,
+        "sensitive", midori_extension_is_prepared (extension),
+        NULL);
 
     g_free (text);
     g_object_unref (extension);
@@ -272,7 +283,7 @@ midori_extensions_treeview_row_activated_cb (GtkTreeView*       treeview,
         gtk_tree_model_get (model, &iter, 0, &extension, -1);
         if (midori_extension_is_active (extension))
             midori_extension_deactivate (extension);
-        else
+        else if (midori_extension_is_prepared (extension))
             g_signal_emit_by_name (extension, "activate", extensions->app);
 
         g_object_unref (extension);
@@ -371,8 +382,7 @@ midori_extensions_popup (GtkWidget*        widget,
                                        midori_extensions_about_activate_cb, FALSE,
                                        extensions);
 
-    sokoke_widget_popup (widget, GTK_MENU (menu),
-                         event, SOKOKE_MENU_POSITION_CURSOR);
+    katze_widget_popup (widget, GTK_MENU (menu), event, KATZE_MENU_POSITION_CURSOR);
 
     g_free (website);
 }
@@ -410,7 +420,7 @@ midori_extensions_button_release_event_cb (GtkWidget*         widget,
 
     if (katze_tree_view_get_selected_iter (GTK_TREE_VIEW (widget), &model, &iter))
     {
-        MidoriExtension *extension;
+        MidoriExtension* extension;
 
         gtk_tree_model_get (model, &iter, 0, &extension, -1);
 
@@ -432,12 +442,12 @@ midori_extensions_cell_renderer_toggled_cb (GtkCellRendererToggle* renderer,
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (extensions->treeview));
     if (gtk_tree_model_get_iter_from_string (model, &iter, path))
     {
-        MidoriExtension *extension;
+        MidoriExtension* extension;
 
         gtk_tree_model_get (model, &iter, 0, &extension, -1);
         if (midori_extension_is_active (extension))
             midori_extension_deactivate (extension);
-        else
+        else if (midori_extension_is_prepared (extension))
             g_signal_emit_by_name (extension, "activate", extensions->app);
 
         g_object_unref (extension);
@@ -460,6 +470,9 @@ midori_extensions_tree_sort_func (GtkTreeModel* model,
     name1 = katze_object_get_string (e1, "name");
     name2 = katze_object_get_string (e2, "name");
 
+    g_object_unref (e1);
+    g_object_unref (e2);
+
     result = g_strcmp0 (name1, name2);
 
     g_free (name1);
